@@ -2,7 +2,11 @@ raSab_bin_fc <-
 function(Z,Y,a,b,Sab,SS=round(sqrt(nrow(Z))))
 {
   E<-Z-a%*%t(rep(1,nrow(Z))) 
-  MEL<-MEU<- -E;MEL[Y==0]<- -Inf;MEU[Y==1]<-Inf;diag(MEU)<-Inf;diag(MEL)<- -Inf
+  MEL<-MEU<- -E
+  MEL[!is.na(Y) & Y==0]<- -Inf
+  MEU[!is.na(Y) & Y==1]<-Inf 
+  MEL[is.na(Y)]<- -Inf ; MEU[is.na(Y)]<- Inf
+#  diag(MEU)<-Inf;diag(MEL)<- -Inf
   lba<-apply(MEL,1,max) 
   lba[is.na(lba)]<- -Inf
   uba<-apply(MEU,1,min) 
